@@ -3,6 +3,7 @@ package aghnet
 import (
 	"io/fs"
 	"net"
+	"net/netip"
 	"path"
 	"strings"
 	"sync/atomic"
@@ -10,6 +11,7 @@ import (
 	"testing/fstest"
 	"time"
 
+	"github.com/AdguardTeam/AdGuardHome/internal/aghchan"
 	"github.com/AdguardTeam/AdGuardHome/internal/aghtest"
 	"github.com/AdguardTeam/golibs/errors"
 	"github.com/AdguardTeam/golibs/netutil"
@@ -135,7 +137,7 @@ func TestNewHostsContainer(t *testing.T) {
 func TestHostsContainer_refresh(t *testing.T) {
 	// TODO(e.burkov):  Test the case with no actual updates.
 
-	ip := net.IP{127, 0, 0, 1}
+	ip := netutil.IPv4Localhost()
 	ipStr := ip.String()
 
 	testFS := fstest.MapFS{"dir/file1": &fstest.MapFile{Data: []byte(ipStr + ` hostname` + nl)}}
@@ -163,27 +165,17 @@ func TestHostsContainer_refresh(t *testing.T) {
 	checkRefresh := func(t *testing.T, want *HostsRecord) {
 		t.Helper()
 
-		var ok bool
-		var upd *netutil.IPMap
-		select {
-		case upd, ok = <-hc.Upd():
-			require.True(t, ok)
-			require.NotNil(t, upd)
-		case <-time.After(1 * time.Second):
-			t.Fatal("did not receive after 1s")
-		}
-
-		assert.Equal(t, 1, upd.Len())
-
-		v, ok := upd.Get(ip)
+		upd, ok := aghchan.MustReceive(hc.Upd(), 1*time.Second)
 		require.True(t, ok)
+		require.NotNil(t, upd)
 
-		require.IsType(t, (*HostsRecord)(nil), v)
+		assert.Len(t, upd, 1)
 
-		rec, _ := v.(*HostsRecord)
+		rec, ok := upd[ip]
+		require.True(t, ok)
 		require.NotNil(t, rec)
 
-		assert.Truef(t, rec.Equal(want), "%+v != %+v", rec, want)
+		assert.Truef(t, rec.equal(want), "%+v != %+v", rec, want)
 	}
 
 	t.Run("initial_refresh", func(t *testing.T) {
@@ -568,13 +560,13 @@ func TestHostsContainer(t *testing.T) {
 }
 
 func TestUniqueRules_ParseLine(t *testing.T) {
-	ip := net.IP{127, 0, 0, 1}
+	ip := netutil.IPv4Localhost()
 	ipStr := ip.String()
 
 	testCases := []struct {
 		name      string
 		line      string
-		wantIP    net.IP
+		wantIP    netip.Addr
 		wantHosts []string
 	}{{
 		name:      "simple",
@@ -589,7 +581,7 @@ func TestUniqueRules_ParseLine(t *testing.T) {
 	}, {
 		name:      "invalid_line",
 		line:      ipStr,
-		wantIP:    nil,
+		wantIP:    netip.Addr{},
 		wantHosts: nil,
 	}, {
 		name:      "invalid_line_hostname",
@@ -604,7 +596,7 @@ func TestUniqueRules_ParseLine(t *testing.T) {
 	}, {
 		name:      "whole_comment",
 		line:      `# ` + ipStr + ` hostname`,
-		wantIP:    nil,
+		wantIP:    netip.Addr{},
 		wantHosts: nil,
 	}, {
 		name:      "partial_comment",
@@ -614,7 +606,7 @@ func TestUniqueRules_ParseLine(t *testing.T) {
 	}, {
 		name:      "empty",
 		line:      ``,
-		wantIP:    nil,
+		wantIP:    netip.Addr{},
 		wantHosts: nil,
 	}}
 
@@ -622,7 +614,7 @@ func TestUniqueRules_ParseLine(t *testing.T) {
 		hp := hostsParser{}
 		t.Run(tc.name, func(t *testing.T) {
 			got, hosts := hp.parseLine(tc.line)
-			assert.True(t, tc.wantIP.Equal(got))
+			assert.Equal(t, tc.wantIP, got)
 			assert.Equal(t, tc.wantHosts, hosts)
 		})
 	}
